# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/xycontroller.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_XYControllerW(object):
    def setupUi(self, XYControllerW):
        XYControllerW.setObjectName("XYControllerW")
        XYControllerW.resize(588, 523)
        self.centralwidget = QtWidgets.QWidget(XYControllerW)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.graphicsView = QtWidgets.QGraphicsView(self.centralwidget)
        self.graphicsView.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.graphicsView.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.graphicsView.setObjectName("graphicsView")
        self.horizontalLayout_3.addWidget(self.graphicsView)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.dial_x = ScalableDial(self.centralwidget)
        self.dial_x.setMinimum(-100)
        self.dial_x.setMaximum(100)
        self.dial_x.setObjectName("dial_x")
        self.verticalLayout.addWidget(self.dial_x)
        spacerItem = QtWidgets.QSpacerItem(20, 30, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout.addItem(spacerItem)
        self.dial_y = ScalableDial(self.centralwidget)
        self.dial_y.setMinimum(-100)
        self.dial_y.setMaximum(100)
        self.dial_y.setObjectName("dial_y")
        self.verticalLayout.addWidget(self.dial_y)
        self.horizontalLayout_3.addLayout(self.verticalLayout)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.line_2 = QtWidgets.QFrame(self.centralwidget)
        self.line_2.setLineWidth(0)
        self.line_2.setMidLineWidth(1)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_2.addWidget(self.line_2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_x_controls = QtWidgets.QLabel(self.centralwidget)
        self.label_x_controls.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_x_controls.setObjectName("label_x_controls")
        self.horizontalLayout.addWidget(self.label_x_controls)
        self.cb_control_x = QtWidgets.QComboBox(self.centralwidget)
        self.cb_control_x.setObjectName("cb_control_x")
        self.horizontalLayout.addWidget(self.cb_control_x)
        self.line = QtWidgets.QFrame(self.centralwidget)
        self.line.setLineWidth(0)
        self.line.setMidLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.horizontalLayout.addWidget(self.line)
        self.label_y_controls = QtWidgets.QLabel(self.centralwidget)
        self.label_y_controls.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_y_controls.setObjectName("label_y_controls")
        self.horizontalLayout.addWidget(self.label_y_controls)
        self.cb_control_y = QtWidgets.QComboBox(self.centralwidget)
        self.cb_control_y.setObjectName("cb_control_y")
        self.horizontalLayout.addWidget(self.cb_control_y)
        self.line_3 = QtWidgets.QFrame(self.centralwidget)
        self.line_3.setLineWidth(0)
        self.line_3.setMidLineWidth(1)
        self.line_3.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.horizontalLayout.addWidget(self.line_3)
        self.cb_smooth = QtWidgets.QCheckBox(self.centralwidget)
        self.cb_smooth.setObjectName("cb_smooth")
        self.horizontalLayout.addWidget(self.cb_smooth)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.scrollArea = QtWidgets.QScrollArea(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scrollArea.sizePolicy().hasHeightForWidth())
        self.scrollArea.setSizePolicy(sizePolicy)
        self.scrollArea.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.scrollArea.setAlignment(QtCore.Qt.AlignCenter)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 1440, 69))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scrollAreaWidgetContents.sizePolicy().hasHeightForWidth())
        self.scrollAreaWidgetContents.setSizePolicy(sizePolicy)
        self.scrollAreaWidgetContents.setMinimumSize(QtCore.QSize(1440, 69))
        self.scrollAreaWidgetContents.setMaximumSize(QtCore.QSize(1440, 69))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.scrollAreaWidgetContents)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.keyboard = PixmapKeyboard(self.scrollAreaWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.keyboard.sizePolicy().hasHeightForWidth())
        self.keyboard.setSizePolicy(sizePolicy)
        self.keyboard.setMinimumSize(QtCore.QSize(1440, 69))
        self.keyboard.setMaximumSize(QtCore.QSize(1440, 69))
        self.keyboard.setObjectName("keyboard")
        self.horizontalLayout_2.addWidget(self.keyboard)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout_2.addWidget(self.scrollArea)
        XYControllerW.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(XYControllerW)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 588, 27))
        self.menubar.setObjectName("menubar")
        self.menu_Settings = QtWidgets.QMenu(self.menubar)
        self.menu_Settings.setObjectName("menu_Settings")
        self.menu_Channels = QtWidgets.QMenu(self.menu_Settings)
        self.menu_Channels.setObjectName("menu_Channels")
        self.menu_File = QtWidgets.QMenu(self.menubar)
        self.menu_File.setObjectName("menu_File")
        XYControllerW.setMenuBar(self.menubar)
        self.act_show_keyboard = QtWidgets.QAction(XYControllerW)
        self.act_show_keyboard.setCheckable(True)
        self.act_show_keyboard.setObjectName("act_show_keyboard")
        self.act_ch_all = QtWidgets.QAction(XYControllerW)
        self.act_ch_all.setObjectName("act_ch_all")
        self.act_ch_01 = QtWidgets.QAction(XYControllerW)
        self.act_ch_01.setCheckable(True)
        self.act_ch_01.setObjectName("act_ch_01")
        self.act_ch_02 = QtWidgets.QAction(XYControllerW)
        self.act_ch_02.setCheckable(True)
        self.act_ch_02.setObjectName("act_ch_02")
        self.act_ch_03 = QtWidgets.QAction(XYControllerW)
        self.act_ch_03.setCheckable(True)
        self.act_ch_03.setObjectName("act_ch_03")
        self.act_ch_04 = QtWidgets.QAction(XYControllerW)
        self.act_ch_04.setCheckable(True)
        self.act_ch_04.setObjectName("act_ch_04")
        self.act_ch_05 = QtWidgets.QAction(XYControllerW)
        self.act_ch_05.setCheckable(True)
        self.act_ch_05.setObjectName("act_ch_05")
        self.act_ch_06 = QtWidgets.QAction(XYControllerW)
        self.act_ch_06.setCheckable(True)
        self.act_ch_06.setObjectName("act_ch_06")
        self.act_ch_07 = QtWidgets.QAction(XYControllerW)
        self.act_ch_07.setCheckable(True)
        self.act_ch_07.setObjectName("act_ch_07")
        self.act_ch_08 = QtWidgets.QAction(XYControllerW)
        self.act_ch_08.setCheckable(True)
        self.act_ch_08.setObjectName("act_ch_08")
        self.act_ch_09 = QtWidgets.QAction(XYControllerW)
        self.act_ch_09.setCheckable(True)
        self.act_ch_09.setObjectName("act_ch_09")
        self.act_ch_10 = QtWidgets.QAction(XYControllerW)
        self.act_ch_10.setCheckable(True)
        self.act_ch_10.setObjectName("act_ch_10")
        self.act_ch_11 = QtWidgets.QAction(XYControllerW)
        self.act_ch_11.setCheckable(True)
        self.act_ch_11.setObjectName("act_ch_11")
        self.act_ch_12 = QtWidgets.QAction(XYControllerW)
        self.act_ch_12.setCheckable(True)
        self.act_ch_12.setObjectName("act_ch_12")
        self.act_ch_13 = QtWidgets.QAction(XYControllerW)
        self.act_ch_13.setCheckable(True)
        self.act_ch_13.setObjectName("act_ch_13")
        self.act_ch_14 = QtWidgets.QAction(XYControllerW)
        self.act_ch_14.setCheckable(True)
        self.act_ch_14.setObjectName("act_ch_14")
        self.act_ch_15 = QtWidgets.QAction(XYControllerW)
        self.act_ch_15.setCheckable(True)
        self.act_ch_15.setObjectName("act_ch_15")
        self.act_ch_16 = QtWidgets.QAction(XYControllerW)
        self.act_ch_16.setCheckable(True)
        self.act_ch_16.setObjectName("act_ch_16")
        self.act_quit = QtWidgets.QAction(XYControllerW)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/16x16/application-exit.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.act_quit.setIcon(icon)
        self.act_quit.setObjectName("act_quit")
        self.act_ch_none = QtWidgets.QAction(XYControllerW)
        self.act_ch_none.setObjectName("act_ch_none")
        self.menu_Channels.addAction(self.act_ch_01)
        self.menu_Channels.addAction(self.act_ch_02)
        self.menu_Channels.addAction(self.act_ch_03)
        self.menu_Channels.addAction(self.act_ch_04)
        self.menu_Channels.addAction(self.act_ch_05)
        self.menu_Channels.addAction(self.act_ch_06)
        self.menu_Channels.addAction(self.act_ch_07)
        self.menu_Channels.addAction(self.act_ch_08)
        self.menu_Channels.addAction(self.act_ch_09)
        self.menu_Channels.addAction(self.act_ch_10)
        self.menu_Channels.addAction(self.act_ch_11)
        self.menu_Channels.addAction(self.act_ch_12)
        self.menu_Channels.addAction(self.act_ch_13)
        self.menu_Channels.addAction(self.act_ch_14)
        self.menu_Channels.addAction(self.act_ch_15)
        self.menu_Channels.addAction(self.act_ch_16)
        self.menu_Channels.addSeparator()
        self.menu_Channels.addAction(self.act_ch_all)
        self.menu_Channels.addAction(self.act_ch_none)
        self.menu_Settings.addAction(self.menu_Channels.menuAction())
        self.menu_Settings.addAction(self.act_show_keyboard)
        self.menu_File.addAction(self.act_quit)
        self.menubar.addAction(self.menu_File.menuAction())
        self.menubar.addAction(self.menu_Settings.menuAction())

        self.retranslateUi(XYControllerW)
        self.act_quit.triggered.connect(XYControllerW.close) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(XYControllerW)

    def retranslateUi(self, XYControllerW):
        _translate = QtCore.QCoreApplication.translate
        XYControllerW.setWindowTitle(_translate("XYControllerW", "XY Controller"))
        self.label_x_controls.setText(_translate("XYControllerW", "X Controls:"))
        self.label_y_controls.setText(_translate("XYControllerW", "Y Controls:"))
        self.cb_smooth.setText(_translate("XYControllerW", "Smooth"))
        self.menu_Settings.setTitle(_translate("XYControllerW", "&Settings"))
        self.menu_Channels.setTitle(_translate("XYControllerW", "Channels"))
        self.menu_File.setTitle(_translate("XYControllerW", "&File"))
        self.act_show_keyboard.setText(_translate("XYControllerW", "Show MIDI &Keyboard"))
        self.act_ch_all.setText(_translate("XYControllerW", "(All)"))
        self.act_ch_01.setText(_translate("XYControllerW", "1"))
        self.act_ch_02.setText(_translate("XYControllerW", "2"))
        self.act_ch_03.setText(_translate("XYControllerW", "3"))
        self.act_ch_04.setText(_translate("XYControllerW", "4"))
        self.act_ch_05.setText(_translate("XYControllerW", "5"))
        self.act_ch_06.setText(_translate("XYControllerW", "6"))
        self.act_ch_07.setText(_translate("XYControllerW", "7"))
        self.act_ch_08.setText(_translate("XYControllerW", "8"))
        self.act_ch_09.setText(_translate("XYControllerW", "9"))
        self.act_ch_10.setText(_translate("XYControllerW", "10"))
        self.act_ch_11.setText(_translate("XYControllerW", "11"))
        self.act_ch_12.setText(_translate("XYControllerW", "12"))
        self.act_ch_13.setText(_translate("XYControllerW", "13"))
        self.act_ch_14.setText(_translate("XYControllerW", "14"))
        self.act_ch_15.setText(_translate("XYControllerW", "15"))
        self.act_ch_16.setText(_translate("XYControllerW", "16"))
        self.act_quit.setText(_translate("XYControllerW", "&Quit"))
        self.act_quit.setShortcut(_translate("XYControllerW", "Esc"))
        self.act_ch_none.setText(_translate("XYControllerW", "(None)"))
from widgets.pixmapkeyboard import PixmapKeyboard
from widgets.scalabledial import ScalableDial
import resources_rc
