# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/carla_osc_connect.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(435, 183)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.group_remote_setup = QtWidgets.QGroupBox(Dialog)
        self.group_remote_setup.setObjectName("group_remote_setup")
        self.gridLayout = QtWidgets.QGridLayout(self.group_remote_setup)
        self.gridLayout.setObjectName("gridLayout")
        self.label_4 = QtWidgets.QLabel(self.group_remote_setup)
        self.label_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 4, 1, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.group_remote_setup)
        self.label_10.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_10.setObjectName("label_10")
        self.gridLayout.addWidget(self.label_10, 0, 1, 1, 1)
        self.sb_udp_port = QtWidgets.QSpinBox(self.group_remote_setup)
        self.sb_udp_port.setMinimum(1024)
        self.sb_udp_port.setMaximum(32767)
        self.sb_udp_port.setObjectName("sb_udp_port")
        self.gridLayout.addWidget(self.sb_udp_port, 4, 2, 1, 1)
        self.sb_tcp_port = QtWidgets.QSpinBox(self.group_remote_setup)
        self.sb_tcp_port.setMinimum(1024)
        self.sb_tcp_port.setMaximum(32767)
        self.sb_tcp_port.setObjectName("sb_tcp_port")
        self.gridLayout.addWidget(self.sb_tcp_port, 2, 2, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.group_remote_setup)
        self.label_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 2, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 60, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 0, 6, 1)
        self.le_host = QtWidgets.QLineEdit(self.group_remote_setup)
        self.le_host.setObjectName("le_host")
        self.gridLayout.addWidget(self.le_host, 0, 2, 1, 2)
        spacerItem1 = QtWidgets.QSpacerItem(20, 60, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 0, 4, 6, 1)
        self.verticalLayout.addWidget(self.group_remote_setup)
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept) # type: ignore
        self.buttonBox.rejected.connect(Dialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Carla Control - Connect"))
        self.group_remote_setup.setTitle(_translate("Dialog", "Remote setup"))
        self.label_4.setText(_translate("Dialog", "UDP Port:"))
        self.label_10.setText(_translate("Dialog", "Remote host:"))
        self.label_3.setText(_translate("Dialog", "TCP Port:"))
import resources_rc
