# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/carla_about_juce.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_JuceAboutW(object):
    def setupUi(self, JuceAboutW):
        JuceAboutW.setObjectName("JuceAboutW")
        JuceAboutW.resize(463, 244)
        self.gridLayout = QtWidgets.QGridLayout(JuceAboutW)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.icon = QtWidgets.QLabel(JuceAboutW)
        self.icon.setMinimumSize(QtCore.QSize(48, 48))
        self.icon.setMaximumSize(QtCore.QSize(48, 48))
        self.icon.setText("")
        self.icon.setPixmap(QtGui.QPixmap(":/48x48/juce.png"))
        self.icon.setObjectName("icon")
        self.verticalLayout.addWidget(self.icon)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.gridLayout.addLayout(self.verticalLayout, 0, 0, 1, 1)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.l_text1 = QtWidgets.QLabel(JuceAboutW)
        self.l_text1.setObjectName("l_text1")
        self.verticalLayout_2.addWidget(self.l_text1)
        spacerItem1 = QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(spacerItem1)
        self.l_text2 = QtWidgets.QLabel(JuceAboutW)
        self.l_text2.setObjectName("l_text2")
        self.verticalLayout_2.addWidget(self.l_text2)
        spacerItem2 = QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(spacerItem2)
        self.l_text3 = QtWidgets.QLabel(JuceAboutW)
        self.l_text3.setWordWrap(True)
        self.l_text3.setObjectName("l_text3")
        self.verticalLayout_2.addWidget(self.l_text3)
        spacerItem3 = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem3)
        self.gridLayout.addLayout(self.verticalLayout_2, 0, 1, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(JuceAboutW)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 1, 0, 1, 2)

        self.retranslateUi(JuceAboutW)
        self.buttonBox.accepted.connect(JuceAboutW.accept) # type: ignore
        self.buttonBox.rejected.connect(JuceAboutW.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(JuceAboutW)

    def retranslateUi(self, JuceAboutW):
        _translate = QtCore.QCoreApplication.translate
        JuceAboutW.setWindowTitle(_translate("JuceAboutW", "About JUCE"))
        self.l_text1.setText(_translate("JuceAboutW", "<b>About JUCE</b>"))
        self.l_text2.setText(_translate("JuceAboutW", "This program uses JUCE version 3.x.x."))
        self.l_text3.setText(_translate("JuceAboutW", "JUCE is an open-source cross-platform C++ application framework for creating high quality desktop and mobile applications.\n"
"\n"
"The core JUCE modules (juce_audio_basics, juce_audio_devices, juce_core and juce_events) are permissively licensed under the terms of the ISC license.\n"
"Other modules are covered by a GNU GPL 3.0 license.\n"
"\n"
"Copyright (C) 2022 Raw Material Software Limited."))
import resources_rc
