# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/carla_settings_driver.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DriverSettingsW(object):
    def setupUi(self, DriverSettingsW):
        DriverSettingsW.setObjectName("DriverSettingsW")
        DriverSettingsW.resize(350, 264)
        DriverSettingsW.setMinimumSize(QtCore.QSize(350, 0))
        self.verticalLayout = QtWidgets.QVBoxLayout(DriverSettingsW)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.cb_samplerate = QtWidgets.QComboBox(DriverSettingsW)
        self.cb_samplerate.setObjectName("cb_samplerate")
        self.gridLayout.addWidget(self.cb_samplerate, 2, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 2, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 1, 2, 1, 1)
        self.cb_device = QtWidgets.QComboBox(DriverSettingsW)
        self.cb_device.setObjectName("cb_device")
        self.gridLayout.addWidget(self.cb_device, 0, 1, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem2, 2, 2, 1, 1)
        self.label_device = QtWidgets.QLabel(DriverSettingsW)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_device.sizePolicy().hasHeightForWidth())
        self.label_device.setSizePolicy(sizePolicy)
        self.label_device.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_device.setObjectName("label_device")
        self.gridLayout.addWidget(self.label_device, 0, 0, 1, 1)
        self.label_buffersize = QtWidgets.QLabel(DriverSettingsW)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_buffersize.sizePolicy().hasHeightForWidth())
        self.label_buffersize.setSizePolicy(sizePolicy)
        self.label_buffersize.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_buffersize.setObjectName("label_buffersize")
        self.gridLayout.addWidget(self.label_buffersize, 1, 0, 1, 1)
        self.cb_buffersize = QtWidgets.QComboBox(DriverSettingsW)
        self.cb_buffersize.setObjectName("cb_buffersize")
        self.gridLayout.addWidget(self.cb_buffersize, 1, 1, 1, 1)
        self.label_samplerate = QtWidgets.QLabel(DriverSettingsW)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_samplerate.sizePolicy().hasHeightForWidth())
        self.label_samplerate.setSizePolicy(sizePolicy)
        self.label_samplerate.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_samplerate.setObjectName("label_samplerate")
        self.gridLayout.addWidget(self.label_samplerate, 2, 0, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.layout_triple_buffer = QtWidgets.QHBoxLayout()
        self.layout_triple_buffer.setObjectName("layout_triple_buffer")
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.layout_triple_buffer.addItem(spacerItem3)
        self.cb_triple_buffer = QtWidgets.QCheckBox(DriverSettingsW)
        self.cb_triple_buffer.setObjectName("cb_triple_buffer")
        self.layout_triple_buffer.addWidget(self.cb_triple_buffer)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.layout_triple_buffer.addItem(spacerItem4)
        self.verticalLayout.addLayout(self.layout_triple_buffer)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem5)
        self.b_panel = QtWidgets.QPushButton(DriverSettingsW)
        self.b_panel.setMinimumSize(QtCore.QSize(32, 0))
        self.b_panel.setObjectName("b_panel")
        self.horizontalLayout_2.addWidget(self.b_panel)
        spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem6)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.layout_restart = QtWidgets.QHBoxLayout()
        self.layout_restart.setObjectName("layout_restart")
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.layout_restart.addItem(spacerItem7)
        self.ico_restart = QtWidgets.QLabel(DriverSettingsW)
        self.ico_restart.setMaximumSize(QtCore.QSize(22, 22))
        self.ico_restart.setText("")
        self.ico_restart.setPixmap(QtGui.QPixmap(":/16x16/dialog-information.svgz"))
        self.ico_restart.setScaledContents(True)
        self.ico_restart.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.ico_restart.setObjectName("ico_restart")
        self.layout_restart.addWidget(self.ico_restart)
        self.label_restart = QtWidgets.QLabel(DriverSettingsW)
        self.label_restart.setObjectName("label_restart")
        self.layout_restart.addWidget(self.label_restart)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.layout_restart.addItem(spacerItem8)
        self.verticalLayout.addLayout(self.layout_restart)
        self.buttonBox = QtWidgets.QDialogButtonBox(DriverSettingsW)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(DriverSettingsW)
        self.buttonBox.accepted.connect(DriverSettingsW.accept) # type: ignore
        self.buttonBox.rejected.connect(DriverSettingsW.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(DriverSettingsW)

    def retranslateUi(self, DriverSettingsW):
        _translate = QtCore.QCoreApplication.translate
        DriverSettingsW.setWindowTitle(_translate("DriverSettingsW", "Driver Settings"))
        self.label_device.setText(_translate("DriverSettingsW", "Device:"))
        self.label_buffersize.setText(_translate("DriverSettingsW", "Buffer size:"))
        self.label_samplerate.setText(_translate("DriverSettingsW", "Sample rate:"))
        self.cb_triple_buffer.setText(_translate("DriverSettingsW", "Triple buffer"))
        self.b_panel.setText(_translate("DriverSettingsW", "Show Driver Control Panel"))
        self.label_restart.setText(_translate("DriverSettingsW", "Restart the engine to load the new settings"))
import resources_rc
