# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/carla_plugin_classic.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_PluginWidget(object):
    def setupUi(self, PluginWidget):
        PluginWidget.setObjectName("PluginWidget")
        PluginWidget.resize(497, 37)
        PluginWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        PluginWidget.setLineWidth(0)
        self.horizontalLayout = QtWidgets.QHBoxLayout(PluginWidget)
        self.horizontalLayout.setContentsMargins(4, 2, 4, 2)
        self.horizontalLayout.setSpacing(2)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.area_left = QtWidgets.QWidget(PluginWidget)
        self.area_left.setObjectName("area_left")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.area_left)
        self.horizontalLayout_3.setContentsMargins(0, 1, 0, 4)
        self.horizontalLayout_3.setSpacing(1)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.b_enable = ScalableButton(self.area_left)
        self.b_enable.setMinimumSize(QtCore.QSize(24, 24))
        self.b_enable.setMaximumSize(QtCore.QSize(24, 24))
        self.b_enable.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/bitmaps/button_off.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_enable.setIcon(icon)
        self.b_enable.setIconSize(QtCore.QSize(24, 24))
        self.b_enable.setCheckable(True)
        self.b_enable.setFlat(True)
        self.b_enable.setObjectName("b_enable")
        self.horizontalLayout_3.addWidget(self.b_enable)
        self.b_gui = ScalableButton(self.area_left)
        self.b_gui.setMinimumSize(QtCore.QSize(24, 24))
        self.b_gui.setMaximumSize(QtCore.QSize(24, 24))
        self.b_gui.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/bitmaps/button_gui.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_gui.setIcon(icon1)
        self.b_gui.setIconSize(QtCore.QSize(24, 24))
        self.b_gui.setCheckable(True)
        self.b_gui.setFlat(True)
        self.b_gui.setObjectName("b_gui")
        self.horizontalLayout_3.addWidget(self.b_gui)
        self.b_edit = ScalableButton(self.area_left)
        self.b_edit.setMinimumSize(QtCore.QSize(24, 24))
        self.b_edit.setMaximumSize(QtCore.QSize(24, 24))
        self.b_edit.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/bitmaps/button_edit.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_edit.setIcon(icon2)
        self.b_edit.setIconSize(QtCore.QSize(24, 24))
        self.b_edit.setCheckable(True)
        self.b_edit.setFlat(True)
        self.b_edit.setObjectName("b_edit")
        self.horizontalLayout_3.addWidget(self.b_edit)
        self.label_name = QtWidgets.QLabel(self.area_left)
        self.label_name.setObjectName("label_name")
        self.horizontalLayout_3.addWidget(self.label_name)
        self.horizontalLayout.addWidget(self.area_left)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.area_right = QtWidgets.QWidget(PluginWidget)
        self.area_right.setObjectName("area_right")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.area_right)
        self.horizontalLayout_2.setContentsMargins(4, 4, 0, 1)
        self.horizontalLayout_2.setSpacing(1)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.led_control = LEDButton(self.area_right)
        self.led_control.setMinimumSize(QtCore.QSize(14, 14))
        self.led_control.setMaximumSize(QtCore.QSize(14, 14))
        self.led_control.setText("")
        self.led_control.setCheckable(True)
        self.led_control.setObjectName("led_control")
        self.horizontalLayout_2.addWidget(self.led_control)
        self.led_midi = LEDButton(self.area_right)
        self.led_midi.setMinimumSize(QtCore.QSize(14, 14))
        self.led_midi.setMaximumSize(QtCore.QSize(14, 14))
        self.led_midi.setText("")
        self.led_midi.setCheckable(True)
        self.led_midi.setObjectName("led_midi")
        self.horizontalLayout_2.addWidget(self.led_midi)
        self.led_audio_in = LEDButton(self.area_right)
        self.led_audio_in.setMinimumSize(QtCore.QSize(14, 14))
        self.led_audio_in.setMaximumSize(QtCore.QSize(14, 14))
        self.led_audio_in.setText("")
        self.led_audio_in.setCheckable(True)
        self.led_audio_in.setObjectName("led_audio_in")
        self.horizontalLayout_2.addWidget(self.led_audio_in)
        self.led_audio_out = LEDButton(self.area_right)
        self.led_audio_out.setMinimumSize(QtCore.QSize(14, 14))
        self.led_audio_out.setMaximumSize(QtCore.QSize(14, 14))
        self.led_audio_out.setText("")
        self.led_audio_out.setCheckable(True)
        self.led_audio_out.setObjectName("led_audio_out")
        self.horizontalLayout_2.addWidget(self.led_audio_out)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setContentsMargins(6, -1, -1, -1)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.peak_in = DigitalPeakMeter(self.area_right)
        self.peak_in.setMinimumSize(QtCore.QSize(150, 0))
        self.peak_in.setMaximumSize(QtCore.QSize(150, 16777215))
        self.peak_in.setObjectName("peak_in")
        self.verticalLayout_2.addWidget(self.peak_in)
        self.peak_out = DigitalPeakMeter(self.area_right)
        self.peak_out.setMinimumSize(QtCore.QSize(150, 0))
        self.peak_out.setMaximumSize(QtCore.QSize(150, 16777215))
        self.peak_out.setObjectName("peak_out")
        self.verticalLayout_2.addWidget(self.peak_out)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.horizontalLayout.addWidget(self.area_right)

        self.retranslateUi(PluginWidget)
        QtCore.QMetaObject.connectSlotsByName(PluginWidget)

    def retranslateUi(self, PluginWidget):
        _translate = QtCore.QCoreApplication.translate
        PluginWidget.setWindowTitle(_translate("PluginWidget", "Frame"))
        self.label_name.setText(_translate("PluginWidget", "Plugin Name"))
from widgets.digitalpeakmeter import DigitalPeakMeter
from widgets.ledbutton import LEDButton
from widgets.scalablebutton import ScalableButton
import resources_rc
