# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/carla_parameter.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_PluginParameter(object):
    def setupUi(self, PluginParameter):
        PluginParameter.setObjectName("PluginParameter")
        PluginParameter.resize(369, 33)
        self.horizontalLayout = QtWidgets.QHBoxLayout(PluginParameter)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.l_name = QtWidgets.QLabel(PluginParameter)
        self.l_name.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.l_name.setObjectName("l_name")
        self.horizontalLayout.addWidget(self.l_name)
        self.widget = ParamSpinBox(PluginParameter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.widget.sizePolicy().hasHeightForWidth())
        self.widget.setSizePolicy(sizePolicy)
        self.widget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.widget.setObjectName("widget")
        self.horizontalLayout.addWidget(self.widget)
        self.l_status = QtWidgets.QLabel(PluginParameter)
        self.l_status.setObjectName("l_status")
        self.horizontalLayout.addWidget(self.l_status)
        self.tb_options = QtWidgets.QToolButton(PluginParameter)
        self.tb_options.setObjectName("tb_options")
        self.horizontalLayout.addWidget(self.tb_options)

        self.retranslateUi(PluginParameter)
        QtCore.QMetaObject.connectSlotsByName(PluginParameter)

    def retranslateUi(self, PluginParameter):
        _translate = QtCore.QCoreApplication.translate
        PluginParameter.setWindowTitle(_translate("PluginParameter", "Form"))
        self.l_name.setText(_translate("PluginParameter", "Parameter Name"))
        self.l_status.setText(_translate("PluginParameter", "TextLabel"))
        self.tb_options.setText(_translate("PluginParameter", "..."))
from widgets.paramspinbox import ParamSpinBox
